/*
  Electron Owl a été créé dans le cadre d'un mémoire du Master Maltt de l'Université de Genève.
  Ainsi, ce code est libre d'utilisation sous la licence CC-BY-NC-SA (Pas d'utilisation commerciale, modification autorisée, libre d'utilisation à la condition de l'attribuer à l'auteur en citant le lien vers cette page)
*/
// Utilise la librairie vis.js
// Dernière modification : 10.08.2017 by JMM
var fs = require('fs');
var visNetworkGraph = null;
function showGraph() {
    //charge le fichier xml de sauvegarde
    const xml = fs.readFileSync('./resources/app/jeu/threads.xml', 'utf8');
    const xmlDoc = $.parseXML(xml);
    const $xml = $(xmlDoc);
    const $root = $xml.find("root");
    // message de prévention
    var count = $root.find('thread[begin="true"]').length;
    if(count === 0){alert("Attention, aucun arbre ne commence au début du jeu !");}
    var nodes = [];
    var edges = [];
    nodes.push({
        id: "START",
        label: "START",
        title: "Début du jeu",
        color: 'salmon',
        shape: 'circle',
    });
    $root.children('thread').each(function () {
        var $tree = $(this);
        nodes.push({
            id: $tree.attr("id"),
            label: "Arbre : " + $tree.attr("name"),
            title: "Arbre : " + $tree.attr('name'),
            color: 'lightgreen',
            shape: 'box'
        });
        var begin = $tree.attr('begin');
        if (begin == "true") { // lie les arbres qui commence au début au cercle "START"
            edges.push({
                from: "START",
                to: $tree.attr("id"),
                arrows: 'to;form',
                color: 'black'
            });
        }
        $tree.find('choice').each(function () {
            var $choice = $(this);
            const id = $choice.attr("id");
            const text = $choice.attr("text"); // affiche le texte en entier si souris placée dessus
            var label = text;
            if (text.length > 13) { // coupe le choix s'il est trop long pour simplifier l'affichage
                label = text.slice(0, 10) + "...";
            }
            nodes.push({
                id: id,
                label: label,
                title: text,
                color: 'lightblue'
            });
            const parentId = $choice.parent().attr("id");
            edges.push({
                from: parentId,
                to: id,
                color: 'black',
                arrows: 'to;form'
            });
            const startThread = $choice.attr('startThread');
            if (startThread && startThread.length > 0) { // affiche les arbres qui commencent à un choix dans l'arbre dudit choix
                startThread.split(/\s+/).forEach(function (item) {
                    const $start = $root.find('thread[threadId="' + item + '"]');
                    nodes.push({
                        id: id + "_" + item,
                        label: "Arbre : " + $start.attr('name'),
                        title: "Arbre : " + $start.attr('name'),
                        color: 'lightgreen',
                        shape: 'box'
                    });
                    edges.push({
                        from: id,
                        to: id + "_" + item,
                        color: 'black',
                        arrows: 'to;form'
                    });
                });
            }
        });
    });
    var data = {
        nodes: new vis.DataSet(nodes),
        edges: new vis.DataSet(edges),
    };
    //la première fois on crée le graphe
    if (visNetworkGraph == null) {
        var container = document.getElementById('graph');
        var options = {
            interaction: {
                dragNodes: false,
                selectable: false
            },
            layout: {
                hierarchical: {
                    direction: 'LR',
                    sortMethod: 'directed',
                    parentCentralization: false
                }
            }
        };
        visNetworkGraph = new vis.Network(container, data, options);
    } else { //sinon on met juste à jour les données
        visNetworkGraph.setData(data);
    }
}
window.onload = function () {
    showGraph();
};