/*
  Electron Owl a été créé dans le cadre d'un mémoire du Master Maltt de l'Université de Genève.
  Ainsi, ce code est libre d'utilisation sous la licence CC-BY-NC-SA (Pas d'utilisation commerciale, modification autorisée, libre d'utilisation à la condition de l'attribuer à l'auteur en citant le lien vers cette page)
*/
// Dernière mise à jour : 10 août 2017, JMM
var fs = require('fs');
// permet d'utiliser electron
const electron = require('electron');
// afin d'afficher la boite de dialogue pour l'import/export
const {
    dialog
} = electron.remote;
// changer ici l'aspect des boutons "supprimer"
const deleteBtnIco = 'deletebtn.svg';
//chemin pour accéder aux fichiers utilisés dans le jeu (présentation, intro, titre, mécanique) depuis la fenêtre Electron
const cheminElectron = "./resources/app/jeu/";
// permet le glisser-déposer des choix en utilisant la bibliothèque Accordion
function addAccordion(query) {
    $(query).accordion({
        heightStyle: "content",
        collapsible: true,
        active: false,
        header: "> div > h3"
    });
}
// index global qui permet que chaque choix ait un identifiant unique
var globalIndex = 0;
window.onload = function () {
    "use strict";
    // permet de savoir si les fenêtres de visualisation des arbres ou pour tester le jeu sont ouvertes ou non, à l'ouverture de la fenêtre principale, elles sont fermées
    var graphWindow = null;
    var gameWindow = null;
    // variable utilisée pour afficher et cacher le menu
    var contextMenuHack = false;
    // crée un nouveau fichier xml (vide le threads.xml en cours)
    $("#new").click(function () {
        if (confirm("Êtes-vous sûr de vouloir recréer un nouveau document ? Exportez votre travail actuel pour ne pas le perdre définitivement !")) {
            fs.writeFileSync(cheminElectron + 'threads.xml', '<root></root>');
            location.reload();
        }
    });
    //charge le fichier xml de sauvegarde
    const xml = fs.readFileSync(cheminElectron + 'threads.xml', 'utf8');
    const xmlDoc = $.parseXML(xml);
    const $xml = $(xmlDoc);
    const $root = $xml.find("root");
    // regénère les id pour être sûr qu'ils soient uniques
    $root.find("thread,choice").each(function () {
        $(this).attr("id", globalIndex++);
    });
    // permet à l'auteur de sauvegarder son projet pour en commencer un autre
    $("#export").click(function () {
        var str = (new XMLSerializer()).serializeToString(xmlDoc);
        var path = dialog.showSaveDialog({
            filters: [{
                name: 'fichier xml',
                extensions: ["xml"]
            }]
        });
        if (path) {
            fs.writeFileSync(path, str);
        }

    });
    // permet à l'auteur d'importer un ancien projet
    $("#import").click(function () {
        var path = dialog.showOpenDialog({
            filters: [{
                name: 'fichier xml',
                extensions: ["xml"]
            }]
        });
        if (path) { // vérifie qu'il s'agisse d'un projet à peu près valide
            const data = fs.readFileSync(path[0], 'utf8');
            if (!data) {
                alert("Impossible de lire le fichier. Veuillez charger un projet valide (.xml)");
                return;
            }
            try {
                if ($($.parseXML(data)).find("root").length == 0) {
                    alert('Votre fichier est vide. Veuillez charger un projet existant ou choisir "Nouveau jeu".');
                    return;
                }
            } catch (e) {
                alert("Impossible de lire le fichier. Veuillez charger un projet valide (.xml)");
                return;
            }
            if (confirm("Êtes-vous sûr de vouloir importer un nouveau document ? Avez-vous exporté votre travail actuel pour ne pas le perdre définitivement ?")) {
                // enregistre ce projet dans threads.xml (écrase le projet précédent)
                fs.writeFileSync(cheminElectron + 'threads.xml', data);
                location.reload();
            }
        }
    });
    // sauvegarde les données dans fichier xml
    var saveThreads = function () {
        var str = (new XMLSerializer()).serializeToString(xmlDoc);
        fs.writeFileSync(cheminElectron + 'threads.xml', str);
        //rafraichit la fenêtre de visualisation si elle est déjà ouverte
        if (graphWindow != null) {
            graphWindow.eval("showGraph();");
        }
    };
    // menu qui s'affiche au clic droit
    require('electron-context-menu')({
        prepend: (params) => {
            var subMenuArray = [];
            var menu = [{
                label: 'Recharger la page', // s'affiche de base dans le menu qui s'affiche au clic droit
                role: 'reload'
            }];
            if (params.pageURL.endsWith('index.html')) {
                return menu; // on ne peut que rafraichir la fenêtre dans la fenêtre du jeu (pour recommencer)
            }
            if (!params.pageURL.endsWith('intaut.html')) {
                return []; // aucun menu ne s'affiche dans les autres fenêtres que l'interface principale (hormis rafraichissement ci-dessus)
            }
            var choiceDiv = null;
            // permet de savoir où se trouve le curseur dans la fenêtre lors du clic
            var element = document.elementFromPoint(params.x, params.y);
            if (element.classList.contains("nomFil")) {
                const threadId = element.getAttribute("threadId");
                var $fil = $root.find('thread[threadId="' + threadId + '"]');
                var begin = $fil.attr('begin') == "true";
                menu.push({
                    label: begin ? 'Ne plus faire commencer l\'arbre au début' : 'Faire commencer l\'arbre au début',
                    click(menuItem, browserWindow, event) {
                        $fil.attr('begin', begin ? "false" : "true");
                        saveThreads();

                        setTreeHeader($fil, $(element));
                    }
                }); // un clic droit sur la barre de titre de l'arbre permet de faire / ne plus faire commencer l'arbre au début (comme l'icône) par le biais d'un menu
            } else {
                // si on clique sur un choix on génère la liste des arbres
                choiceDiv = element.closest('.sortable');
                if (choiceDiv != null) {
                    // permet de savoir dans quel arbre on se trouve
                    const currentThreadId = choiceDiv.closest('.fil').id.split('_')[1]; //l'id est sous forme "fil_<threadId>"
                    //cherche les infos de référence des choix déjà présentes dans le xml
                    var choiceNode = $root.find('#' + choiceDiv.id);
                    var startThread = choiceNode.attr('startThread');
                    // appliqué pour chaque arbre
                    $root.children('thread').each(function () {
                        var $tree = $(this);
                        const name = $tree.attr("name");
                        const threadId = $tree.attr("threadId");
                        if (currentThreadId == threadId)
                            return; // on ne va pas afficher son propre arbre
                        var remove = false;
                        var startThreadArray = []; //tableau vide par défaut
                        if (startThread) { //si l'attribut est défini dans le xml il est converti en tableau
                            startThreadArray = startThread.split(/\s+/);
                        }
                        if (startThreadArray.indexOf(threadId) != -1) { //au cas où l'array contiendrait déjà notre arbre
                            remove = true;
                        }
                        var subMenuEntry = {
                            label: threadId + ": " + name + (remove ? ' (retirer)' : ''),
                            click(menuItem, browserWindow, event) {
                                if (remove) { //enlève cet arbre du document répertoriant les arbres qui commencent à ce choix
                                    startThreadArray.splice(startThreadArray.indexOf(threadId), 1); //enlève l'élément du tableau
                                } else { //ajoute cet arbre dans l'array comme commençant à ce choix
                                    startThreadArray.push(threadId);
                                }
                                //met la valeur dans le xml de sauvegarde
                                choiceNode.attr("startThread", startThreadArray.join(' '));
                                saveThreads();
                                setChoiceHeader(choiceNode, $(choiceDiv.firstChild));
                            }
                        };
                        subMenuArray.push(subMenuEntry);
                    });
                }
            }
            menu.push({
                label: 'Faire commencer l\'arbre...',
                submenu: subMenuArray,
                visible: choiceDiv != null
            });

            if (contextMenuHack) {
                contextMenuHack = false;
                return subMenuArray;
            }
            return menu;
        }
    });
    // permet d'afficher les zones dédiées au nom de l'auteur, du jeu et au texte de présentation
    function afficherTexte(zone, nodeName, textareaName, classTextArea, textHolder) {
        if ($root.find(nodeName).length == 0) {
            // si le fichier xml est vierge, l'élément est créé
            $root.append('<' + nodeName + '/>');
        }
        var node = $root.find(nodeName);
        //ajoute la barre de titre
        $("#" + zone).before("<h3>" + textareaName + "</h3>");
        var textArea;
        if (node.text() == "") { // si l'élément est vide, met un texte d'aide, sinon affiche le texte
            textArea = $('<textarea class="' + classTextArea + '" placeholder="' + textHolder + '"></textarea>');
        } else {
            textArea = $('<textarea class="' + classTextArea + '">' + node.text() + '</textarea>');
        }
        $('#' + zone).append(textArea);
        // fonction pour sauvegarder les modifications apportées
        textArea.change(function () {
            node.text(this.value);
            saveThreads();
        });
    }
    // affiche le titre du jeu, permet de le modifier
    function afficherTitre() {
        var zone = "zoneTitre";
        var textTitle = "Le titre";
        var textHolder = "Le titre de votre jeu";
        afficherTexte(zone, "title", textTitle, "choix", textHolder);
    }
    // affiche le nom de l'auteur du jeu, permet de le modifier
    function afficherAuteur() {
        var zone = "zoneAuteur";
        var textTitle = "Nom de l'auteur";
        var textHolder = 'Votre nom ou le nom que vous souhaiteriez voir apparaître après la mention Créé par';
        afficherTexte(zone, "author", textTitle, "choix", textHolder);
    }
    // affiche le texte de présentation du jeu, permet de le modifier
    function afficherPresent() {
        var zone = "zonePresent";
        var textTitle = "Le texte de présentation";
        var textHolder = "Une petite présentation de votre jeu motivant à jouer à votre jeu !";
        afficherTexte(zone, "present", textTitle, "narra", textHolder);
    }
    //affiche les arbres et les choix
    function showThread($thread) {
        const name = $thread.attr("name");
        const threadId = $thread.attr("threadId");
        $('#thread_' + threadId).sortable({
            axis: "y",
            handle: "h3",
            items: ".sortable",
            connectWith: ".children", // permet de bouger vers le parent
            delay: 250, // évite de bouger sans faire exprès quand on clique dessus
            containment: '#thread_' + threadId, // empêche de bouger l'élément en dehors de l'arbre
            stop: function (event, ui) {
                const id = ui.item[0].id;
                var movedNode = $root.find('#' + id);
                // enlève le choix de sa place de départ
                movedNode.detach();
                // s'il y a un élément avant l'endroit où se trouve le choix, il s'insère simplement après cet élément
                if (ui.item[0].previousSibling) {
                    const previousId = ui.item[0].previousSibling.id;
                    // insère le choix après previousId
                    $root.find('#' + previousId).after(movedNode);
                } else { // s'il s'agit du premier élément de la liste, on va chercher le parent et utiliser prepend
                    // le parent est de classe children, son id est de la forme 'children_<id>' où <id> est ce qu'on cherche
                    const parentId = ui.item[0].parentNode.id.split('_')[1];
                    //insère le choix en premier dans la liste
                    $root.find('#' + parentId).prepend(movedNode);
                }
                saveThreads();
                // Rafraichit accordion pour prendre en compte le nouvel ordre
                $(".children").accordion("refresh");
            }
        });
        // div qui contiendra tout un arbre
        var threadDiv = $('#thread_' + threadId);
        // bouton pour supprimer cet arbre
        var threadDeleteBtn = $('<img src=' + deleteBtnIco + ' title="Supprimer cet arbre" width="2%" align="right">');
        // bouton pour faire commencer ou cesser de faire commencer cet arbre au début du jeu
        var startThreadBeginBtn = $('<img src="atstart.svg" title="Faire / Ne plus faire commencer cet arbre au début du jeu" width="50px"><br>');
        threadDiv.append(startThreadBeginBtn);
        //permet de changer le nom de l'arbre
        threadDiv.append('<label>Nom de l\'arbre : </label>');
        var name_input;
        if (name == "") { // si vide, message explicatif, sinon nom de l'arbre
            name_input = $('<input type="text" placeholder="Nom de l\'arbre" maxlength="40" size="21">');
        } else {
            name_input = $('<input type="text" value="' + name + '" maxlength="40" size="21">');
        }
        threadDiv.append(name_input);
        threadDiv.prev().append(threadDeleteBtn);
        // fonction pour sauvegarder les modifications apportées au texte
        name_input.change(function () {
            $thread.attr("name", this.value);
            saveThreads();
            //met à jour le titre de l'arbre
            setTreeHeader($thread, threadDiv.prev());
        });
        // sauvegarder le changement commence / ne commence pas au début
        startThreadBeginBtn.on('click',
            function () {
                var $fil = $root.find('thread[threadId="' + threadId + '"]');
                var begin = $fil.attr('begin') == "true";
                $fil.attr('begin', begin ? "false" : "true");
                saveThreads();
                setTreeHeader($thread, threadDiv.prev());
            });
        // supprimer l'arbre
        threadDeleteBtn.on('click', function () {
            var yes = confirm('Êtes-vous sûr de vouloir supprimer l\'arbre ' + threadId + ' : ' + $thread.attr("name") + ' ?');
            if (yes) {
                // supprime l'arbre de l'interface et du fichier threads.xml
                threadDiv.prev().remove();
                threadDiv.remove();
                $thread.remove();

                //supprime l'arbre de tous les choix qui le commençaient
                $root.find("choice[startThread~=" + threadId + "]").each(function () {
                    var choice = $(this);
                    var startThreadArray = choice.attr("startThread").split(/\s+/);
                    startThreadArray.splice(startThreadArray.indexOf(threadId), 1);
                    choice.attr("startThread", startThreadArray.join(' '));
                    const id = choice.attr('id');
                    var header = $('#' + id).children('h3');
                    // met à jour la barre de titre du choix
                    setChoiceHeader(choice, header);
                });
                saveThreads();
            }
        });
        // Affiche l'introduction de l'arbre
        const narrationDivId = threadId + '_thread_narr';
        var narrationDiv = $('<div id="' + narrationDivId + '"></div>');
        threadDiv.append(narrationDiv);
        if ($thread.children('narr').length < 1) {
            //crée l'élément si inexistant
            $thread.append('<narr></narr>');
        }
        var narr = $($thread.children('narr')[0]);
        narrationDiv.append("<h4>Introduction de l'arbre :</h4>");
        var narra_textArea;
        if (narr.text() == "") { // message si vide
            narra_textArea = $('<textarea class="narra" placeholder="Introduction de l\'arbre"></textarea>');
        } else {
            narra_textArea = $('<textarea class="narra">' + narr.text() + '</textarea>');
        }
        narrationDiv.append(narra_textArea);
        // sauvegarde les modifications apportées au texte
        narra_textArea.change(function () {
            narr.text(this.value);
            saveThreads();
        });
        // lance l'affichage
        show($thread, threadDiv);
    }
    // texte qui s'affiche dans la barre de titre des arbres
    function setTreeHeader(node, header) {
        //préserve les infos ajoutées par accordion
        var cache = header.children();
        header.text('Arbre : ' + node.attr('name'));
        var begin = node.attr("begin");
        if (begin == "true") {
            header.text(header.text() + '    [Démarre au début]');
        }
        //restaure les infos ajoutées par accordion
        header.prepend(cache);
    }
    // texte qui s'affiche dans la barre de titre des choix
    function setChoiceHeader(node, header) {
        var cache = header.children();
        header.text('Choix joueur : ' + node.attr('text').slice(0, 30));
        // si le choix lance un arbre, cela s'affiche dans la barre de titre
        var startThread = node.attr("startThread");
        if (startThread) {
            var list = '';
            startThread.split(/\s+/).forEach(function (item) {
                var name = $root.find('thread[threadId="' + item + '"]').attr('name');
                if (list.length > 0) list += ', ';
                list += item + ':' + name;
            });
            header.text(header.text() + '    [Lance : ' + list + ']');
        }
        header.prepend(cache);
    }
    // affiche le contenu des choix dans l'interface
    function show(sourceNode, destNode) {
        // chaque choix est compris dans un div unique
        const childrenId = "children_" + sourceNode.attr("id");
        var childrenDiv = $('#' + childrenId);
        if (childrenDiv.length == 0) { //crée le div que s'il n'existe pas déjà
            childrenDiv = $('<div id="' + childrenId + '" class ="children"></div>');
            destNode.append(childrenDiv);
        }
        sourceNode.children('choice').each(function () {
            const currentNode = $(this);
            const nodeId = currentNode.attr("id");
            const text = currentNode.attr('text');
            const choiceDivId = 'choice_' + nodeId;
            if ($('#' + choiceDivId).length == 0) { //crée le div que s'il n'existe pas déjà
                var childDiv = $('<div id="' + choiceDivId + '" class="childDiv"></div>');
                if (childrenDiv.parents().length % 2 == 0) {
                    childDiv = $('<div id="' + choiceDivId + '" class="childDiv odd"></div>');
                } else {
                    childDiv = $('<div id="' + choiceDivId + '" class="childDiv even"></div>');
                }
                // ce div contient le h3 et le childDiv pour pouvoir les déplacer ensemble. Son Id est celui du node dans le fichier de sauvegarde
                var movableDiv;
                if (childrenDiv.parents().length % 2 == 0) { // deux classes différentes pour faciliter l'affichage
                    movableDiv = $('<div id="' + nodeId + '" class="sortable odd"></div>');
                } else {
                    movableDiv = $('<div id="' + nodeId + '" class="sortable even"></div>');
                }
                // barre de titre sur laquelle on clique pour ouvrir/fermer le choix avec accordion
                var header = $("<h3></h3>");
                movableDiv.append(header);
                movableDiv.append(childDiv);
                childrenDiv.append(movableDiv);
                setChoiceHeader(currentNode, header);
                // bouton de suppression rattaché à ce choix
                var deleteButton = $('<img src=' + deleteBtnIco + ' title="Supprimer ce choix et ses sous-choix" width="2%" align="right">');
                header.append(deleteButton);
                deleteButton.click(function () {
                    if (confirm("Voulez-vous vraiment supprimer ce choix et tous ses sous-choix ? (Choix : " + choice_textArea.val() + ")")) {
                        currentNode.remove(); //enlève le choix du xml
                        saveThreads();
                        movableDiv.remove();
                    }
                });
                // bouton pour faire commencer un arbre à ce choix
                var startTreeBtn = $('<img src="atachoice.svg" title ="Faire / Ne plus faire commencer un autre arbre ici" width="50px"><br>');
                startTreeBtn.click(event, function () {
                    // permet d'afficher le menu déroulant
                    contextMenuHack = true;
                    electron.remote.BrowserWindow.getFocusedWindow().webContents.sendInputEvent({
                        type: 'mouseUp',
                        x: event.x,
                        y: event.y,
                        button: 'right',
                        clickCount: 1
                    });
                });
                childDiv.append(startTreeBtn);
                // narration qui suit le choix
                var choice_textArea;
                if (text == "") { // texte de base si élément vide
                    choice_textArea = $('<textarea class="narra" placeholder="Nouveau choix"></textarea>');
                } else {
                    choice_textArea = $('<textarea class="narra">' + text + '</textarea>');
                }
                childDiv.append(choice_textArea);
                // sauvegarde les modifications apportées au texte
                choice_textArea.change(function () {
                    currentNode.attr('text', this.value);
                    saveThreads();
                    //met à jour le titre du choix
                    setChoiceHeader(currentNode, header);
                });
                // affiche la narration dans un div personnel
                const narrationDivId = choiceDivId + '_narr';
                var narrationDiv = $('<div id="' + narrationDivId + '"></div>');
                childDiv.append(narrationDiv);
                if (currentNode.children('narr').length < 1) { //crée l'élément si inexistant
                    currentNode.append('<narr></narr>');
                }
                var narr = $(currentNode.children('narr')[0]);
                var narra_textArea;
                if (narr.text() == "") { // texte d'aide si vide
                    narra_textArea = $('<textarea class="narra" placeholder="Narration enclenchée à ce choix"></textarea>');
                } else {
                    narra_textArea = $('<textarea class="narra">' + narr.text() + '</textarea>');
                }
                narrationDiv.append('<h4>Narration qui suit :</h4>');
                narrationDiv.append(narra_textArea);
                // sauvegarde les modifications
                narra_textArea.change(function () {
                    narr.text(this.value);
                    saveThreads();
                });
                //récursion, pour afficher tout ce qui existe
                show(currentNode, childDiv);
            }
        });
        if (sourceNode[0].parentElement != null) { //div inutile pour que sortable marche quand il n'y a aucun child
            childrenDiv.append('<div id="dropTarget" class="sortable"></div>');
        }
        // bouton pour créer un nouveau choix
        var createButton = $('<img src="createchoice.svg" title="Créer choix suivant" width="40px"><br>');
        destNode.append(createButton);
        createButton.on('click', function () {
            // enlève le bouton
            $(this).remove();
            // enlève le div dropTarget pour qu'il soit recréé au bon endroit
            childrenDiv.find('> #dropTarget').each(function () {
                $(this).remove();
            });
            // crée un nouvel id unique
            const newId = globalIndex++;
            //ajoute le nouveau choix dans le xml et sauvegarde le fichier
            sourceNode.append('<choice text="" id="' + newId + '"></choice>');
            saveThreads();

            // réaffiche la nouvelle partie de l'arbre
            show(sourceNode, destNode);
            addAccordion("#choice_" + newId + " .children");
            $(".children").accordion("refresh");
        });
    }
    // bouton pour créer un nouvel arbre
    var createTreeButton = $('<br><img src="createtree.svg" title="Créer un nouvel arbre" width="50px">');
    $("#createTreeArea").append(createTreeButton);
    // ajoute le bouton pour créer un nouvel arbre à la suite des arbres existants
    function addButtonCreateNextTree() {
        createTreeButton.on('click', function () {
            var name = "";
            var threadId = 0;
            //trouve un nouvel id pour cet arbre qui n'est pas encore utilisé
            while ($root.find('thread[threadId="' + threadId + '"]').length > 0) {
                threadId++;
            }
            //ajoute l'arbre dans le xml
            $root.append('<thread id="' + globalIndex++ + '" threadId="' + threadId + '" name="' + name + '"></thread>');
            saveThreads();
            // ajoute avant le bouton
            $("#treeArea").append('<h2 class="nomFil" threadId="' + threadId + '">Arbre : ' + name + '</h2><div id="thread_' + threadId + '" class="fil"></div>');
            showThread($root.find('thread[threadId="' + threadId + '"]'));
            addAccordion("#thread_" + threadId + " .children");
            $("#treeArea").accordion("refresh");
        });
    }
    // affiche les arbres
    function showTrees() {
        $root.find("thread").each(function () {
            var $thread = $(this);
            const threadId = $thread.attr("threadId");
            var header = $('<h2 class="nomFil" threadId="' + threadId + '"></h2>');
            $("#treeArea").append(header);
            $("#treeArea").append('<div id="thread_' + threadId + '" class="fil"></div>');
            setTreeHeader($thread, header);
            showThread($thread);
        });
        addButtonCreateNextTree();
        $("#treeArea").accordion({
            heightStyle: "content",
            collapsible: true,
            active: false
        });
        addAccordion(".children");
    }
    // lance la visualisation des arbres dans une nouvelle fenêtre
    $("#visualisation").click(function () {
        if (graphWindow == null || graphWindow.closed) {
            graphWindow = window.open(`file://${__dirname}/graph.html`);
        } else {
            graphWindow.focus();
        }
    });
    // crée le contenu qui s'affichera à l'écran quand les zones ci-dessus seront apparentes
    afficherTitre();
    afficherAuteur();
    afficherPresent();
    showTrees();
    $("#zonePresentationGenerale").accordion({
        heightStyle: "content",
        collapsible: true,
        active: false,
        header: "h3"
    });
    // lance le test du jeu dans une nouvelle fenêtre
    $("#jeu").on('click', function () {
        var count = $root.find('thread[begin="true"]').length;
        if (count === 0) { // message de prévention si aucun arbre ne commence au début
            alert("Attention, aucun arbre ne commence au début du jeu ! Veuillez faire commencer un arbre au début du jeu avant de le tester.");
        } else {
            if (gameWindow == null || gameWindow.closed) {
                gameWindow = window.open('./jeu/index.html');
            } else { // si la fenêtre était ouverte, on la ferme pour la relancer
                gameWindow.close();
                gameWindow = window.open('./jeu/index.html');
            }
        }
    });
};